import pytest
from pnlp.pmag import MagicDict, get_unique_fn


def test_magic():
    tmd = MagicDict()
    tmd["a"]["b"]["c"] = 1
    assert tmd["a"]["b"]["c"] == 1


def test_magic_set_get():
    d = MagicDict()
    d["a"]["b"] = 2
    assert d.a.b == 2


def test_magic_reverse():
    dx = {1: "a", 2: "a", 3: "a", 4: "b"}
    assert MagicDict.reverse(dx) == {"a": [1, 2, 3], "b": 4}



@pytest.mark.parametrize("inp,oup,level", [
    ("a.md", "a.md", 0),
    ("a.md", "a.md", 1),
    ("a.md", "a.md", 10),
    ("a/b.md", "a_b.md", 0),
    ("a/b.md", "a_b.md", 1),
    ("a/b.md", "a_b.md", 10),
    ("a/b/c.md", "a_b_c.md", 0),
    ("a/b/c.md", "b_c.md", 1),
    ("a/b/c.md", "a_b_c.md", 10),
    ("/a/b/c.md", "a_b_c.md", 0),
    ("/a/b/c.md", "b_c.md", 1),
    ("/a/b/c.md", "a_b_c.md", 10),
])
def test_get_unique_fn(inp, level, oup):
    res = get_unique_fn(inp, level)
    assert res == oup



import pytest
from pnlp.penh import swap, SentenceLevelSampler, TokenLevelSampler
from pnlp.pcut import psubsent, cut_part, cut_zhchar


def test_swap_middle():
    lst = [1, 2, 3, 4, 5]
    new = swap(lst, 2, 0, 4)
    assert new == [1, 3, 2, 4, 5] or new == [1, 2, 4, 3, 5]


def test_swap_start():
    lst = [1, 2, 3]
    new = swap(lst, 0, 0, 2)
    assert new == [2, 1, 3]


def test_swap_end():
    lst = [1, 2, 3]
    new = swap(lst, 2, 0, 2)
    assert new == [1, 3, 2]


def cut_words(text: str) -> list:
    return [
        '人', '为什么', '活着', '？',
        '生而为', '人', '必须', '要', '有', '梦想', '！',
        '还要', '有', '尽可能', '多', '的', '精神', '体验', '。']


def cut_wps(text: str) -> list:
    return [
        ('人', 'n'), ('为什么', 'r'), ('活着', 'v'), ('？', 'w'),
        ('生而为人', 'v'), ('必须', 'd'), ('要', 'v'),
        ('有', 'v'), ('梦想', 'n'), ('！', 'w'),
        ('还要', 'v'), ('有', 'v'), ('尽可能', 'd'), ('多', 'a'),
        ('的', 'u'), ('精神', 'n'), ('体验', 'vn'), ('。', 'w')
    ]


def test_token_level_sampler():
    tls = TokenLevelSampler()
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    res = tls.make_samples(text)
    assert type(res) == dict
    assert len(res) == 4


def test_token_level_sampler_none():
    tls = TokenLevelSampler(types=[])
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    res = tls.make_samples(text)
    assert res == {}


def test_token_level_sampler_none_text():
    tls = TokenLevelSampler()
    text = ""
    res = tls.make_samples(text)
    assert res == {}


def test_token_level_sampler_single_sent():
    tls = TokenLevelSampler()
    text = "人为什么活着？"
    res = tls.make_samples(text)
    assert len(res) == 4


def test_token_level_sampler_independent_sampling():
    tls = TokenLevelSampler()
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    tokens = cut_zhchar(text)
    res = tls.independent_sampling(tokens)
    assert type(res) == list
    assert len(res) == 3


def test_token_level_sampler_dependent_sampling():
    tls = TokenLevelSampler()
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    tokens = cut_zhchar(text)
    res = tls.dependent_sampling(tokens)
    assert type(res) == list
    assert type(res[0]) == str


def test_token_level_sampler_delete():
    tls = TokenLevelSampler(types=["delete"])
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    res = tls.make_samples(text)
    assert type(res) == dict
    assert len(res) == 2


def test_token_level_sampler_swap():
    tls = TokenLevelSampler(types=["swap"])
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    res = tls.make_samples(text)
    assert type(res) == dict
    assert len(res) == 2


def test_token_level_sampler_insert():
    tls = TokenLevelSampler(types=["insert"])
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    res = tls.make_samples(text)
    assert type(res) == dict
    assert len(res) == 2


def test_token_level_sampler_token_spliter():
    tls = TokenLevelSampler()
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    res = tls.make_samples(text, cut_words)
    assert len(res) == 4


def test_token_level_sampler_token_pos_spliter():
    tls = TokenLevelSampler()
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    res = tls.make_samples(text, cut_wps)
    assert len(res) == 4


def test_token_level_sampler_delete_sampling():
    tls = TokenLevelSampler()
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    tokens = cut_words(text)
    res = tls.delete_sampling(tokens, [2])
    assert type(res) == list
    assert len(res) + 1 == len(tokens)


def test_token_level_sampler_insert_sampling():
    tls = TokenLevelSampler()
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    tokens = cut_words(text)
    res = tls.insert_sampling(tokens, [2, 6])
    assert type(res) == list
    assert len(res) - 2 == len(tokens)


def test_token_level_sampler_swap_sampling():
    tls = TokenLevelSampler()
    text = "人为什么活着？生而为人必须要有梦想！还要有尽可能多的精神体验。"
    tokens = cut_words(text)
    res = tls.swap_sampling(tokens, [5, 8])
    assert type(res) == list
    assert len(res) == len(tokens)


def test_sentence_level_sampler():
    sls = SentenceLevelSampler()
    text = "我爱你。你爱我。"
    res = sls.make_samples(text)
    assert type(res) == dict
    assert len(res) == 4


def test_sentence_level_sampler_none():
    sls = SentenceLevelSampler([])
    text = "我爱你。你爱我。"
    assert sls.make_samples(text) == {}


def test_sentence_level_sampler_single_sent():
    sls = SentenceLevelSampler()
    text = "我爱你。"
    assert len(sls.make_samples(text)) == 4


def test_sentence_level_sampler_none_text():
    sls = SentenceLevelSampler()
    text = ""
    assert sls.make_samples(text) == {}


def test_sentence_level_sampler_independent_sampling():
    sls = SentenceLevelSampler()
    text = "写代码。写好代码。"
    text_list = cut_part(text, psubsent)
    res = sls.independent_sampling(text_list)
    assert type(res) == list
    assert len(res) == 3
    assert len(res[0]) == 1
    assert len(res[1]) == 2
    assert len(res[2]) == 3


def test_sentence_level_sampler_dependent_sampling():
    sls = SentenceLevelSampler()
    text = "写代码。多写代码。写好代码。"
    text_list = cut_part(text, psubsent)
    res = sls.dependent_sampling(text_list)
    assert type(res) == list
    assert len(res) == 3


def test_sentence_level_sampler_insert():
    sls = SentenceLevelSampler(types=["insert"])
    text = "我爱你。你爱我。NLP 很有意思。简洁最重要。"
    res = sls.make_samples(text)
    assert len(res) == 2


def test_sentence_level_sampler_delete():
    sls = SentenceLevelSampler(types=["delete"])
    text = "我爱你。你爱我。NLP 很有意思。简洁最重要。"
    res = sls.make_samples(text)
    assert len(res) == 2


def test_sentence_level_sampler_swap():
    sls = SentenceLevelSampler(types=["swap"])
    text = "我爱你。你爱我。NLP 很有意思。简洁最重要。"
    res = sls.make_samples(text)
    assert len(res) == 2



from pnlp.piop import read_file, read_lines, read_json, read_yaml, read_csv, read_pickle, read_file_to_list_dict
from pnlp.piop import write_file, write_json, write_pickle, check_dir, write_list_dict_to_file
from pnlp.pcut import cut_sentence, cut_sub_sentence, cut_zhchar, cut_part, combine_bucket
from pnlp.pcut import psent, psubsent

from pnlp.piop import Reader, Dict
from pnlp.ptxt import Regex, Text, Length
from pnlp.pnorm import NumNorm
from pnlp.penh import TokenLevelSampler, SentenceLevelSampler
from pnlp.ptrans import pick_entity_from_bio_labels, generate_uuid
from pnlp.pmag import MagicDict, get_unique_fn
from pnlp.stopwords import StopWords
from pnlp.stopwords import chinese_stopwords, english_stopwords

from pnlp.utils import pstr, concurring, divide2int, run_in_new_thread
from pnlp.utils import generate_batches_by_num, generate_batches_by_size


num_norm = NumNorm()
reg = Regex()
reader = Reader()
tlsampler = TokenLevelSampler()
slsampler = SentenceLevelSampler()


__title__ = "pnlp"
__version__ = "0.4.15"
__author__ = "Yam"
__license__ = "Apache-2.0"
__copyright__ = "Copyright 2019, 2020, 2021, 2022, 2023, 2024 Yam"
__all__ = [
    "Reader",
    "Text", "Regex", "Length",
    "MagicDict",
    "NumNorm",
    "StopWords",
    "TokenLevelSampler", "SentenceLevelSampler"
]



import re
import os
from functools import partial
import math
import itertools
import pytest
import multiprocessing as mp

from pnlp.utils import pstr, concurring, generate_batches_by_num
from pnlp.utils import run_in_new_thread
from pnlp.piop import read_file, write_file


def test_pstr1():
    s1 = pstr("123")
    s2 = "1"
    assert s1 - s2 == "23"


def test_pstr2():
    s1 = pstr("123")
    s2 = "123"
    assert s1 - s2 == ""


def test_pstr3():
    s1 = pstr("123")
    s2 = "234"
    assert s1 - s2 == "1"


def test_pstr4():
    s1 = pstr("123")
    s2 = "456"
    assert s1 - s2 == "123"


def test_pstr5():
    s1 = pstr("")
    s2 = "456"
    assert s1 - s2 == ""


def test_generate_batches():
    lst = range(100)
    res = list(generate_batches_by_num(lst, 10))
    assert len(res) == 10
    assert len(res[0]) == 10


def is_prime(x):
    if x < 2:
        return False
    for i in range(2, int(math.sqrt(x)) + 1):
        if x % i == 0:
            return False
    return True


def test_concurring_default():

    @concurring
    def get_primes(lst):
        res = []
        for i in lst:
            if is_prime(i):
                res.append(i)
        return res
    lst = list(range(100))
    res = get_primes(lst)
    res = list(res)
    assert len(res) == mp.cpu_count()
    res = list(itertools.chain(*res))
    assert len(res) == 25


@pytest.mark.parametrize(
    "type",
    ["thread_pool", "process_pool", "thread_executor", "thread"])
@pytest.mark.parametrize("max_workers", [1, 2, 4, 7, 10])
def test_concurring_with_parameters(type, max_workers):

    @concurring(type=type, max_workers=max_workers)
    def get_primes(lst):
        res = []
        for i in lst:
            if is_prime(i):
                res.append(i)
        return res
    lst = list(range(100))
    res = get_primes(lst)
    res = list(res)
    assert len(res) == max_workers
    res = list(itertools.chain(*res))
    assert len(res) == 25


def test_concurring_invalid_type():

    @concurring(type="invalid")
    def get_primes(lst):
        res = []
        for i in lst:
            if is_prime(i):
                res.append(i)
        return res
    lst = list(range(100))
    try:
        res = get_primes(lst)
    except Exception as err:
        assert "invalid" in str(err)


def test_concurring_invalid_workers():

    try:
        @concurring(max_workers=0)
        def get_primes(lst):
            res = []
            for i in lst:
                if is_prime(i):
                    res.append(i)
            return res
    except Exception as err:
        assert "0" in str(err)



def test_run_in_thread():
    file = "run_in_new_thread.txt"
    
    def func(file, a, b, c):
        write_file(file, list(map(str, [a, b, c])))

    run_in_new_thread(func, file, 1, 2, 3)
    import time
    time.sleep(1)

    assert os.path.exists(file)
    os.remove(file)


def test_run_in_thread_kwargs():
    kwargs = {
        "b": 2,
        "c": 3,
    }

    def func(a, b, c):
        return a + b + c

    func = partial(func, **kwargs)

    assert 6 == func(1)




import re
import pytest

from pnlp.pcut import cut_sentence, cut_zhchar, combine_bucket, cut_sub_sentence


def test_text2zhchar1():
    text = "我喜欢你。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "。"]


def test_text2zhchar2():
    text = "我 喜欢 你。"
    ret = cut_zhchar(text)
    assert ret == ["我", " ", "喜", "欢", " ", "你", "。"]


def test_text2zhchar3():
    text = "我喜欢like你。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "like", "你", "。"]


def test_text2zhchar4():
    text = "我喜欢你233。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "233", "。"]


def test_text2zhchar5():
    text = "我喜欢你3.14。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "3.14", "。"]


def test_text2zhchar6():
    text = "我喜欢你100%。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "100%", "。"]


def test_text2zhchar7():
    text = "我喜欢你2/3。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "2/3", "。"]


def test_text2zhchar8():
    text = "我喜欢你-2。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "-2", "。"]


def test_text2zhchar9():
    text = "我喜欢你2、3。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "2", "、", "3", "。"]


def test_text2zhchar10():
    text = "我喜欢你。。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "。", "。"]


def test_text2zhchar11():
    text = "我喜欢你A-B。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "A", "-", "B", "。"]


def test_text2zhchar12():
    text = "我喜欢你A_B。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "A", "_", "B", "。"]


def test_text2zhchar13():
    text = "我喜欢你C++。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "C", "+", "+", "。"]


def test_text2zhchar14():
    text = "我喜欢你R&B。"
    ret = cut_zhchar(text)
    assert ret == ["我", "喜", "欢", "你", "R", "&", "B", "。"]


def test_text2zhchar15():
    text = "#我喜欢你。"
    ret = cut_zhchar(text)
    assert ret == ["#", "我", "喜", "欢", "你", "。"]


def test_text2zhchar16():
    text = "我 喜欢 你。"
    ret = cut_zhchar(text, remove_blank=True)
    assert ret == ["我", "喜", "欢", "你", "。"]


def test_text2zhchar17():
    text = "我 love you."
    ret = cut_zhchar(text, remove_blank=True)
    assert ret == ["我", "love", "you", "."]


def test_text2zhchar18():
    text = "lo-ve."
    ret = cut_zhchar(text, remove_blank=True)
    assert ret == ['lo', '-', 've', '.']


def test_text2zhchar19():
    text = "v-.f "
    ret = cut_zhchar(text, remove_blank=True)
    assert ret == ['v', '-', '.', 'f']


def test_text2zhchar20():
    text = "-1.2."
    ret = cut_zhchar(text)
    assert ret == ['-1.2', '.']


def test_text2zhchar21():
    text = "1-2-3-"
    ret = cut_zhchar(text)
    assert ret == ['1-2-3-']


def test_text2zhchar22():
    text = "-1-2-3"
    ret = cut_zhchar(text)
    assert ret == ['-1-2-3']


def test_text2zhchar23():
    text = "1.2.3"
    ret = cut_zhchar(text)
    assert ret == ['1.2.3']


def test_text2zhchar24():
    text = "1..2"
    ret = cut_zhchar(text)
    assert ret == ['1..2']


def test_text2zhchar25():
    text = "1.2..."
    ret = cut_zhchar(text)
    assert ret == ['1.2', '.', '.', '.']


def test_text2zhchar26():
    text = "1...2..."
    ret = cut_zhchar(text)
    assert ret == ['1...2', '.', '.', '.']


def test_text2zhchar27():
    text = """
    x..x 1.2, -1.23 lo-.ve.. -1-2-3- 2-2. -1.2. 3/5 1.2.3 1..2 2% 3.5% -2.0%
    """
    ret = cut_zhchar(text, remove_blank=True)
    assert ret == [
        'x', '.', '.', 'x',
        '1.2', ',', '-1.23',
        'lo', '-', '.', 've', '.', '.',
        '-1-2-3-', '2-2', '.',
        '-1.2', '.', '3/5',
        '1.2.3', '1..2',
        '2%', '3.5%', '-2.0%'
    ]


def test_text2sent1():
    text = "我喜欢你，你呢？哈哈，我不告诉你。"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret[0] == "我喜欢你，你呢？"
    assert ret[1] == "哈哈，我不告诉你。"


def test_text2sent2():
    text = "我喜欢你，你呢！哈哈，我不告诉你"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret[0] == "我喜欢你，你呢！"
    assert ret[1] == "哈哈，我不告诉你"


def test_text2sent3():
    text = "我喜欢你，「哈哈」。我不告诉你~~~"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret[0] == "我喜欢你，「哈哈」。"
    assert ret[1] == "我不告诉你~~~"


def test_text2sent4():
    text = "我喜欢你，“哈哈”.我不告诉你……"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret[0] == "我喜欢你，“哈哈”."
    assert ret[1] == "我不告诉你……"


def test_text2sent5():
    text = "我喜欢你，“哈哈” 我不告诉你；"
    ret = cut_sentence(text)
    assert len(ret) == 1
    assert ret[0] == "我喜欢你，“哈哈” 我不告诉你；"


def test_text2sent6():
    text = "我喜欢你，“哈哈。” 我不告诉你!"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret[0] == "我喜欢你，“哈哈。”"
    assert ret[1] == " 我不告诉你!"


def test_text2sent7():
    text = "我喜欢你(haha). 我不告诉你～"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret[0] == "我喜欢你(haha)."
    assert ret[1] == " 我不告诉你～"


def test_text2sent8():
    text = "我喜欢你, “哈哈……”。“我不告诉你.”"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret[0] == "我喜欢你, “哈哈……”。"
    assert ret[1] == "“我不告诉你.”"


def test_text2sent9():
    text = "我喜欢你&“哈哈？”“我不告诉你”"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret[0] == "我喜欢你&“哈哈？”"
    assert ret[1] == "“我不告诉你”"


def test_text2sent10():
    text = "我喜欢你，"
    ret = cut_sentence(text)
    assert len(ret) == 1
    assert ret[0] == "我喜欢你，"


def test_text2sent11():
    text = "我喜欢你"
    ret = cut_sentence(text)
    assert len(ret) == 1
    assert ret[0] == "我喜欢你"


def test_text2sent12():
    text = "我喜欢\n你"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret == ["我喜欢\n", "你"]


def test_text2sent13():
    text = "我喜欢。\n你"
    ret = cut_sentence(text)
    assert len(ret) == 3
    assert ret == ["我喜欢。", "\n", "你"]


def test_text2sent14():
    text = "我喜欢。\n.你"
    ret = cut_sentence(text)
    assert len(ret) == 3
    assert ret == ["我喜欢。", "\n", ".你"]


def test_text2sent15():
    text = "我喜欢\n.你"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret == ["我喜欢\n", ".你"]


def test_text2sent16():
    text = "我喜欢 .你"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret == ["我喜欢 .", "你"]


def test_text2sent17():
    text = "我喜欢　.你"
    ret = cut_sentence(text)
    assert len(ret) == 2
    assert ret == ["我喜欢　.", "你"]


@pytest.fixture
def parts():
    return [
        '习近平指出',
        '中方不仅维护中国人民生命安全和身体健康',
        '也维护世界人民生命安全和身体健康',
        '我们本着公开',
        '透明',
    ]


def test_combine_bucket1(parts):
    ret = combine_bucket(parts.copy(), 5)
    assert ret == parts
    ret = combine_bucket(parts.copy(), 10)
    assert ret == [
        '习近平指出',
        '中方不仅维护中国人民生命安全和身体健康',
        '也维护世界人民生命安全和身体健康',
        '我们本着公开透明',
    ]


def test_combine_bucket2(parts):
    ret = combine_bucket(parts.copy(), 5, truncate=True)
    assert ret == [
        '习近平指出',
        '中方不仅维',
        '也维护世界',
        '我们本着公',
        '透明'
    ]
    ret = combine_bucket(parts.copy(), 10, truncate=True)
    assert ret == [
        '习近平指出',
        '中方不仅维护中国人民',
        '也维护世界人民生命安',
        '我们本着公开透明',
    ]


def test_combine_bucket3(parts):
    ret = combine_bucket(parts.copy(), 5, truncate=True, keep_remain=True)
    assert ret == [
        '习近平指出',
        '中方不仅维',
        '护中国人民',
        '生命安全和',
        '身体健康',
        '也维护世界',
        '人民生命安',
        '全和身体健',
        '康',
        '我们本着公',
        '开',
        '透明'
    ]
    ret = combine_bucket(parts.copy(), 10, truncate=True, keep_remain=True)
    assert ret == [
        '习近平指出',
        '中方不仅维护中国人民',
        '生命安全和身体健康',
        '也维护世界人民生命安',
        '全和身体健康',
        '我们本着公开透明',
    ]



@pytest.mark.parametrize("inp, expected", [
    ("1,2,3,4", ["1,", "2,", "3,", "4"]),
    ("2/5是0.4。", ["2/5是0.4。"]),
    ("2/5是0.4.", ["2/5是0.4."]),
    ("2除以8等于0.25。", ["2除以8等于0.25。"]),
])
def test_cut_subsent(inp, expected):
    res = cut_sub_sentence(inp)
    assert res == expected


from pnlp.stopwords import StopWords



def test_stopwords():
    sw = StopWords()
    assert type(sw.zh) == set
    assert type(sw.en) == set
    assert sw.zh_len > 0
    assert sw.en_len > 0


def test_custom_stopwords():
    sw = StopWords("tests/piop_data/b.txt")
    assert type(sw.stopwords) == set
    assert len(sw.stopwords) == 3


import os
import pytest
import types

from pnlp.piop import write_json, write_file
from pnlp.piop import Reader, read_file, read_lines, read_json, read_yaml, read_csv
from pnlp.piop import read_file_to_list_dict, write_list_dict_to_file
from pnlp.piop import check_dir

DATA_PATH = os.path.join('tests', 'piop_data')


@pytest.fixture(params=['*.md', '*.txt', '*.data', 'f*.*', '*c.*'])
def get_Reader_path_match_res(request):
    res = []
    reader = Reader(request.param)
    for line in reader(DATA_PATH):
        res.append(line)
    return res


def test_Reader_path_match(get_Reader_path_match_res):
    assert len(get_Reader_path_match_res) == 9
    assert get_Reader_path_match_res[0].lid == 0
    assert get_Reader_path_match_res[-1].lid == 2


def test_Reader_file():
    res = []
    reader = Reader()
    for line in reader(os.path.join(DATA_PATH, 'a.md')):
        res.append(line)
    assert len(res) == 3
    assert res[0].text == 'line 1 in a.'


def test_Reader_gen_files():
    paths = Reader.gen_files(DATA_PATH, '*.md')
    assert isinstance(paths, types.GeneratorType)
    assert len(list(paths)) == 3


def test_Reader_gen_files_with_regex():
    paths = Reader.gen_files(DATA_PATH, "(md)|(txt)", True)
    assert isinstance(paths, types.GeneratorType)
    assert len(list(paths)) == 6


def test_Reader_gen_articles():
    paths = Reader.gen_files(DATA_PATH, '*.txt')
    articles = Reader.gen_articles(paths)
    assert isinstance(articles, types.GeneratorType)
    assert len(list(articles)) == 3


def test_Reader_gen_flines():
    paths = Reader.gen_files(DATA_PATH, '*.txt')
    articles = Reader.gen_articles(paths)
    lines = Reader.gen_flines(articles)
    assert isinstance(lines, types.GeneratorType)
    assert len(list(lines)) == 9


def test_Reader_gen_plines():
    lines = Reader.gen_plines(os.path.join(DATA_PATH, 'b.txt'))
    assert isinstance(lines, types.GeneratorType)
    assert len(list(lines)) == 3


@pytest.fixture
def get_read_data():
    return os.path.join(DATA_PATH, 'c.data')


def test_read_file(get_read_data):
    data = read_file(get_read_data)
    assert data == 'line 1 in c.\nline 2 in c.\nline 3 in c.'
    assert type(data) == str


@pytest.mark.parametrize("count", [0, 1, 2, -1])
def test_read_lines(get_read_data, count):
    data = read_lines(get_read_data, count=count)
    if count != -1:
        assert len(data) == count
    else:
        assert data == ['line 1 in c.', 'line 2 in c.', 'line 3 in c.']
    assert type(data) == list


def test_read_json():
    data = read_json(os.path.join(DATA_PATH, 'json.json'))
    assert type(data) == dict
    assert data == {
        "json1": "this is line 1",
        "json2": "这是第二行。"
    }


def test_read_yaml():
    data = read_yaml(os.path.join(DATA_PATH, 'yaml.yaml'))
    assert type(data) == dict
    assert data == {'元旦': ['新年快乐', '元旦快乐', '节日快乐'],
                    '周末': ['周末快乐！', '周末愉快！']}


def test_read_csv():
    data = read_csv(os.path.join(DATA_PATH, 'csv.csv'))
    assert type(data) == list
    assert data == [['id', 'title'], ['1', 'title1'], ['2', 'title2']]


def test_read_file_to_list_dict():
    data = read_file_to_list_dict(os.path.join(DATA_PATH, "list_dict.json"))
    assert type(data) == list
    assert type(data[0]) == dict


def test_write_json():
    data = {"outjson1": "this is line 1.",
            "outjson2": "这是第二行。"}
    write_json(os.path.join(DATA_PATH, 'outjson.json'),
               data, indent=4, ensure_ascii=False)


def test_write_file():
    data = ['line 1 of outfile.', '这是 outfile 的第二行。']
    write_file(os.path.join(DATA_PATH, 'outfile.file'), data)


def test_write_list_dict_to_file():
    data = [{"name": "Yam", "age": 20}]
    write_list_dict_to_file(os.path.join(DATA_PATH, "outfile.listdict"), data)


def test_check_dir():
    assert check_dir(DATA_PATH) is None



import pytest

from pnlp.pnorm import NumNorm

c2a = NumNorm()


def test_chinese2arabic1():
    s = "一亿三千万"
    assert c2a.zh2num(s) == 130000000


def test_chinese2arabic2():
    s = "一万五千六百三十八"
    assert c2a.zh2num(s) == 15638


def test_chinese2arabic3():
    s = "壹仟两百"
    assert c2a.zh2num(s) == 1200


def test_chinese2arabic4():
    s = "壹仟两百零三"
    assert c2a.zh2num(s) == 1203


def test_chinese2arabic5():
    s = "壹仟两百一十五"
    assert c2a.zh2num(s) == 1215


def test_chinese2arabic6():
    s = "壹仟两百九十"
    assert c2a.zh2num(s) == 1290


def test_chinese2arabic7():
    s = "十一"
    assert c2a.zh2num(s) == 11


def test_chinese2arabic8():
    s = "八十八"
    assert c2a.zh2num(s) == 88


def test_chinese2arabic9():
    s = "三"
    assert c2a.zh2num(s) == 3


def test_chinese2arabic10():
    s = "两百五十"
    assert c2a.zh2num(s) == 250


def test_chinese2arabic11():
    s = "两百"
    assert c2a.zh2num(s) == 200


def test_chinese2arabic12():
    s = "两百零五"
    assert c2a.zh2num(s) == 205


def test_chinese2arabic13():
    s = "两百二十五"
    assert c2a.zh2num(s) == 225


def test_chinese2arabic14():
    s = "二十万五千"
    assert c2a.zh2num(s) == 205000


def test_chinese2arabic15():
    s = "两百三十九万四千八百二十三"
    assert c2a.zh2num(s) == 2394823


def test_chinese2arabic16():
    s = "一千三百万"
    assert c2a.zh2num(s) == 13000000


def test_chinese2arabic17():
    s = "万"
    assert c2a.zh2num(s) == "万"


def test_chinese2arabic18():
    s = "亿"
    assert c2a.zh2num(s) == "亿"


def test_chinese2arabic19():
    s = "千"
    assert c2a.zh2num(s) == "千"


def test_chinese2arabic20():
    s = "百"
    assert c2a.zh2num(s) == "百"


def test_chinese2arabic21():
    s = "零"
    assert c2a.zh2num(s) == 0


def test_arabic2chinese1():
    num = 0
    assert c2a.num2zh(num) == "零"


def test_arabic2chinese2():
    num = 1
    assert c2a.num2zh(num) == "一"


def test_arabic2chinese3():
    num = 10
    assert c2a.num2zh(num) == "一十"


def test_arabic2chinese4():
    num = 12
    assert c2a.num2zh(num) == "一十二"


def test_arabic2chinese5():
    num = 22
    assert c2a.num2zh(num) == "二十二"


def test_arabic2chinese6():
    num = 100
    assert c2a.num2zh(num) == "一百"


def test_arabic2chinese7():
    num = 101
    assert c2a.num2zh(num) == "一百零一"


def test_arabic2chinese8():
    num = 110
    assert c2a.num2zh(num) == "一百一十"


def test_arabic2chinese9():
    num = 112
    assert c2a.num2zh(num) == "一百一十二"


def test_arabic2chinese10():
    num = 1000
    assert c2a.num2zh(num) == "一千"


def test_arabic2chinese11():
    num = 1001
    assert c2a.num2zh(num) == "一千零一"


def test_arabic2chinese12():
    num = 1011
    assert c2a.num2zh(num) == "一千零一十一"


def test_arabic2chinese13():
    num = 1101
    assert c2a.num2zh(num) == "一千一百零一"


def test_arabic2chinese14():
    num = 1010
    assert c2a.num2zh(num) == "一千零一十"


def test_arabic2chinese15():
    num = 1100
    assert c2a.num2zh(num) == "一千一百"


def test_arabic2chinese16():
    num = 1110
    assert c2a.num2zh(num) == "一千一百一十"


def test_arabic2chinese17():
    num = 1111
    assert c2a.num2zh(num) == "一千一百一十一"


def test_arabic2chinese18():
    num = 100000
    assert c2a.num2zh(num) == "一十万"


def test_arabic2chinese19():
    num = 110000
    assert c2a.num2zh(num) == "一十一万"


def test_arabic2chinese20():
    num = 1000000
    assert c2a.num2zh(num) == "一百万"


def test_arabic2chinese21():
    num = 1010000
    assert c2a.num2zh(num) == "一百零一万"


def test_arabic2chinese22():
    num = 1100000
    assert c2a.num2zh(num) == "一百一十万"


def test_arabic2chinese23():
    num = 1110000
    assert c2a.num2zh(num) == "一百一十一万"


def test_arabic2chinese24():
    num = 100000000
    assert c2a.num2zh(num) == "一亿"


def test_arabic2chinese25():
    num = 110000000
    assert c2a.num2zh(num) == "一亿一千万"


def test_arabic2chinese26():
    num = 111000000
    assert c2a.num2zh(num) == "一亿一千一百万"


def test_arabic2chinese27():
    num = 101000000
    assert c2a.num2zh(num) == "一亿零一百万"


def test_arabic2chinese28():
    num = 1000000000000
    assert c2a.num2zh(num) == "一万亿"


def test_arabic2chinese29():
    num = 1100000000000
    assert c2a.num2zh(num) == "一万一千亿"


def test_arabic2chinese30():
    # 一兆一亿
    num = 11000000000000
    assert c2a.num2zh(num) == "超大"


def test_arabic2chinese31():
    num = 1110011
    assert c2a.num2zh(num) == "一百一十一万零一十一"


def test_arabic2chinese_money1():
    num = 112
    assert c2a.num2zh(num).to_money() == "壹佰壹拾贰"


def test_arabic2chinese_money2():
    num = 1111
    assert c2a.num2zh(num).to_money() == "壹仟壹佰壹拾壹"


def test_arabic2chinese_money3():
    num = 1010000
    assert c2a.num2zh(num).to_money() == "壹佰零壹萬"




import re
import pytest

from pnlp.ptxt import Text, Regex, Length
reg = Regex()


@pytest.mark.parametrize("inp, expected", [
    ("1.t", ""),
    ("1,t", "1"),
    (",1", "1"),
])
def test_regex_pnum(inp, expected):
    match = reg.pnum.search(inp)
    if match:
        res = match.group()
    else:
        res = ""
    assert res == expected


@pytest.fixture(params=reg.patnames)
def get_regex(request):
    return reg.patdict[request.param]


def test_regex_well(get_regex):
    assert isinstance(get_regex, re.Pattern)


@pytest.fixture(params=reg.patnames)
def get_patten(request):
    return [request.param]


def test_Text_extract(get_patten):
    text = "这是，测试fdsf234*(&( 返回类型的文本。"
    res = Text(get_patten).extract(text)
    assert isinstance(res, dict)
    assert isinstance(res.mats, list)
    assert isinstance(res.locs, list)


def test_Text_clean(get_patten):
    text = "这是，测试fdsf234*(&( 返回类型的文本。"
    res = Text(get_patten).clean(text)
    assert isinstance(res, str)


def test_pattern_string_invalid():
    try:
        Text(["XX"])
    except Exception as e:
        assert "built-in" in str(e)


def test_pattern_invalid():
    try:
        Text([lambda x: x])
    except Exception as e:
        assert "RE" in str(e)


@pytest.fixture
def text_chi():
    text = "你好。jefj*(&-1)这是中文测试！"
    return text


def test_Text_extract_chi(text_chi):
    res = Text(['chi']).extract(text_chi)
    assert "".join(res.mats) == "你好这是中文测试"
    assert res.text == "你好这是中文测试"


def test_Text_clean_chi(text_chi):
    res = Text(['chi']).clean(text_chi)
    assert res == "。jefj*(&-1)！"


@pytest.fixture
def text_pun():
    text = "你好，这是标点,.!;<>()符号测试。"
    return text


def test_Text_extract_pun(text_pun):
    res = Text(['nwn']).extract(text_pun)
    assert "".join(res.mats) == "，,.!;<>()。"
    assert res.text == "，,.!;<>()。"


def test_Text_clean_pun(text_pun):
    res = Text(['nwn']).clean(text_pun)
    assert res == "你好这是标点符号测试"


@pytest.fixture
def text_whi():
    text = "你好，这是空白 \t\n符号测试。"
    return text


def test_Text_extract_whi(text_whi):
    res = Text(['whi']).extract(text_whi)
    assert "".join(res.mats) == " \t\n"
    assert res.text == " \t\n"


def test_Text_clean_whi(text_whi):
    res = Text(['whi']).clean(text_whi)
    assert res == "你好，这是空白符号测试。"


@pytest.fixture
def text_nwh():
    text = "你好，这是非空白 \t\n符号测试。"
    return text


def test_Text_extract_nwh(text_nwh):
    res = Text(['nwh']).extract(text_nwh)
    assert "".join(res.mats) == "你好，这是非空白符号测试。"
    assert res.text == "你好，这是非空白符号测试。"


def test_Text_clean_nwh(text_nwh):
    res = Text(['nwh']).clean(text_nwh)
    assert res == " \t\n"


@pytest.fixture
def text_wnb():
    text = "你好，这是词与word数字number测试。"
    return text


def test_Text_extract_wnb(text_wnb):
    res = Text(['wnb']).extract(text_wnb)
    assert "".join(res.mats) == "你好这是词与word数字number测试"
    assert res.text == "你好这是词与word数字number测试"


def test_Text_clean_wnb(text_wnb):
    res = Text(['wnb']).clean(text_wnb)
    assert res == "，。"


@pytest.fixture
def text_nwn():
    text = "你好，这是非词或word数字number测试。"
    return text


def test_Text_extract_nwn(text_nwn):
    res = Text(['nwn']).extract(text_nwn)
    assert "".join(res.mats) == "，。"
    assert res.text == "，。"


def test_Text_clean_nwn(text_nwn):
    res = Text(['nwn']).clean(text_nwn)
    assert res == "你好这是非词或word数字number测试"


@pytest.fixture
def text_eng():
    text = "你好，这#￥是英文English测试。"
    return text


def test_Text_extract_eng(text_eng):
    res = Text(['eng']).extract(text_eng)
    assert "".join(res.mats) == "English"
    assert res.text == "English"


def test_Text_clean_eng(text_eng):
    res = Text(['eng']).clean(text_eng)
    assert res == "你好，这#￥是英文测试。"


@pytest.fixture
def text_num():
    text = "你好，这#￥是数字2, +2, -2, 2.1, -2.2, 1/5, 2:3, -2/5, 2%, 2.5%测试。"
    return text


def test_Text_extract_num(text_num):
    res = Text(['num']).extract(text_num)
    assert "".join(res.mats) == "2+2-22.1-2.21/52:3-2/52%2.5%"
    assert res.text == "2+2-22.1-2.21/52:3-2/52%2.5%"


def test_Text_clean_num(text_num):
    res = Text(['num']).clean(text_num)
    assert res == "你好，这#￥是数字, , , , , , , , , 测试。"


@pytest.fixture
def text_pic():
    text = "你好，这#￥是![p1](https://xxx.jpeg)图片![](yyy.png)测试https://z.jpg。"
    return text


def test_Text_extract_pic(text_pic):
    res = Text(['pic']).extract(text_pic)
    assert "".join(
        res.mats) == "![p1](https://xxx.jpeg)![](yyy.png)https://z.jpg"
    assert res.text == "![p1](https://xxx.jpeg)![](yyy.png)https://z.jpg"


def test_Text_clean_pic(text_pic):
    res = Text(['pic']).clean(text_pic)
    assert res == "你好，这#￥是图片测试。"


@pytest.fixture
def text_lnk():
    text = "你好，www.g.com，这#￥是链接[link](https://yam.gift)测试http://yam.gift。"
    return text


def test_Text_extract_lnk(text_lnk):
    res = Text(['lnk']).extract(text_lnk)
    assert "".join(
        res.mats) == "www.g.com[link](https://yam.gift)http://yam.gift"
    assert res.text == "www.g.com[link](https://yam.gift)http://yam.gift"


def test_Text_clean_lnk(text_lnk):
    res = Text(['lnk']).clean(text_lnk)
    assert res == "你好，，这#￥是链接测试。"


def test_markdown_link_with_whitespace_in_title():
    text = """啊[Download | View](https://www.altova.com/xmlspy-xml-editor/download/)"""
    res = Text(['lnk']).clean(text)
    assert res == "啊"


@pytest.fixture
def text_emj():
    text = "你好，这#￥是表情😁测试😜🌹。"
    return text


def test_Text_extract_emj(text_emj):
    res = Text(['emj']).extract(text_emj)
    assert "".join(res.mats) == "😁😜🌹"
    assert res.text == "😁😜🌹"


def test_Text_clean_emj(text_emj):
    res = Text(['emj']).clean(text_emj)
    assert res == "你好，这#￥是表情测试。"


@pytest.fixture
def text_len():
    text = "这是https://www.yam.gift长度测试，《 》*)FSJfdsjf😁![](http://xx.jpg)。233."
    return text


def test_Text_len_all(text_len):
    res = Length(text_len).len_all
    assert res == 64


def test_Text_len_nwh(text_len):
    res = Length(text_len).len_nwh
    assert res == 63


def test_Text_len_chi(text_len):
    res = Length(text_len).len_chi
    assert res == 6


def test_Text_len_wnb(text_len):
    res = Length(text_len).len_wnb
    assert res == 41


def test_Text_len_pun(text_len):
    res = Length(text_len).len_pun
    assert res == 14


def test_Text_len_eng(text_len):
    res = Length(text_len).len_eng
    assert res == 32


def test_Text_len_num(text_len):
    res = Length(text_len).len_num
    assert res == 3


if __name__ == '__main__':
    print(reg.patnames)
